#define USEFOG

float4x4 viewProjMatrix : register (c0);
float4x4 worldMatrix : register (c4);
float4 worldEye: register (c8);
float4 fogData : register (c9);
float4 texMod : register (c11); // tex0.u=x, tex0.v=y, tex1.u=z, tex1.v=w
float4 dirLight : register (c14);
float4 sunLight : register (c13);

struct VS_OUTPUT
{
   float4 Pos  : POSITION;
   float4 Diff : COLOR0;
#ifdef USEFOG
   float  Fog  : FOG;
#endif   
#if NUMTEX == 1
   float2 Tex  : TEXCOORD0;
#else   
   float4 Tex  : TEXCOORD0;
#endif
   float Exposition : TEXCOORD1;   
};

#if SCROLLTEX == 1

  #if NUMTEX == 2
    VS_OUTPUT SSTR_L30_noShadeModTexT0T1 (float4 inPos : POSITION, float4 inColor : COLOR0, float2 inTex : TEXCOORD0, float2 inTex1 : TEXCOORD1)
  #else  
    VS_OUTPUT SSTR_L30_noShadeModTexT0 (float4 inPos : POSITION, float4 inColor : COLOR0, float2 inTex : TEXCOORD0)
  #endif  

#else

  #if NUMTEX == 2
    VS_OUTPUT SSTR_L30_noShadeT0T1 (float4 inPos : POSITION, float4 inColor : COLOR0, float2 inTex : TEXCOORD0, float2 inTex1 : TEXCOORD1)
  #elif NUMTEX == 1
    VS_OUTPUT SSTR_L30_noShadeT0 (float4 inPos : POSITION, float4 inColor : COLOR0, float2 inTex : TEXCOORD0)
  #else  
    VS_OUTPUT SSTR_L30_noShade (float4 inPos : POSITION, float4 inColor : COLOR0, float2 inTex : TEXCOORD0)
  #endif  
  
#endif  
{
  VS_OUTPUT Out;
  
  // Transform vertex position into view space:
  Out.Pos = mul (viewProjMatrix, inPos);

//exposition
float lightDist = distance(normalize(mul(viewProjMatrix, normalize(dirLight)).xyz), float3(0.0, 0.0, 1.0)); //0.0 - looking at sun, 2.0 - looking away from sun
lightDist = pow(lightDist / 2.0, 1.3);
Out.Exposition = lerp(-1.3, lightDist * -1.3 - 0.5, saturate(sunLight.g * 2.0 - 0.3));


#ifdef USEFOG
  // compute fog
  float3 eyeVec = mul (worldMatrix, inPos) + worldEye;
  Out.Fog.x = (fogData[1] - (length (eyeVec.xyz) * fogData[0]));
#endif
  
  // Propagate color and texture coordinates:
  Out.Diff = inColor;
  
#if SCROLLTEX == 1

  Out.Tex.xy = inTex + texMod.xy;
 #if NUMTEX == 2
  Out.Tex.zw = inTex1 + texMod.zw;
 #endif   

#else
  
  Out.Tex.xy = inTex;
 #if NUMTEX == 2
  Out.Tex.zw = inTex1;
 #endif   

#endif
  return (Out);
}








