
float4x4 viewProjMatrix : register (c0);
float4x4 worldMatrix : register (c4);
float4 worldEye : register (c8);
float4 fogData : register (c9);
float4 dirLight : register (c14);
float4 worldLight    : register (c2);

int4 numOmniLight : register (i0); // (count, init, stride, 0)
float4 omniPosRad[64]   : register (c72);
float4 omniRGBDelta[64] : register (c136);

struct VS_OUTPUT
{
   float4 Pos    : POSITION;
   float4 Diff   : COLOR0;
   float  Fog    : FOG;
   float2 Tex    : TEXCOORD0; // base
   float4 Spec   : TEXCOORD1; // spec and track map
   float4 Normal : TEXCOORD2; // world space vertex normal + sign of reflection
   float3 EyeVec : TEXCOORD3; // world space eye vec

   float3 Basis1 : TEXCOORD4; // tangent space eye vec
   float3 Basis2 : TEXCOORD5; // tangent space eye vec
   float3 Basis3 : TEXCOORD6; // tangent space eye vec

   float4 Omni   : COLOR1;
};


  VS_OUTPUT SSTR_CURB_T0SPECBUMPSKID02 (float4 inPos : POSITION, float4 inNorm : NORMAL, float4 inColor : COLOR0,
                                        float2 inTex0 : TEXCOORD0, float2 inTangent : TEXCOORD1,
                                        float2 inBinorm : TEXCOORD2, float2 inSkidmarks : TEXCOORD3)
{
  VS_OUTPUT Out;
  
  // Transform vertex position into view space:
  Out.Pos = mul (viewProjMatrix, inPos);

  float3 worldNorm = mul (worldMatrix, inNorm.xyz);
  Out.Normal.xyz = worldNorm;
  Out.Normal.w = worldMatrix[1][1];
  
  float3 worldPos = mul (worldMatrix, inPos);
  float3 eyeVec = (worldEye.xyz + worldPos.xyz);
  Out.EyeVec = eyeVec;
  
  float3 inTangent3;
  float3 inBinorm3;
  inTangent3.xz = inTangent;
  inTangent3.y = 0.0;
  inBinorm3.xz = inBinorm;
  inBinorm3.y = 0.0;

  float3 worldS = mul (worldMatrix, inTangent3.xyz);
  float3 worldT = mul (worldMatrix, inBinorm3.xyz);
  float3 worldSxT = mul (worldMatrix, inNorm.xyz);

  Out.Basis1.x = worldS.x;
  Out.Basis1.y = worldT.x;
  Out.Basis1.z = worldSxT.x;
  Out.Basis2.x = worldS.y;
  Out.Basis2.y = worldT.y;
  Out.Basis2.z = worldSxT.y;
  Out.Basis3.x = worldS.z;
  Out.Basis3.y = worldT.z;
  Out.Basis3.z = worldSxT.z;

  Out.Fog = (fogData[1] - (length (eyeVec.xyz) * fogData[0]));

  // Compute omnilights
  Out.Omni = float4(0,0,0,0);
  for (int i=0; i<numOmniLight.x; i++)
  {
    float intens;
    float3 omniToVert = omniPosRad[i].xyz - worldPos.xyz;
    float mag = dot (omniToVert.xyz, omniToVert.xyz);
    // check if vert in range
    if ( mag < omniPosRad[i].w) // using square distances
      if ((intens = dot (worldNorm, normalize (omniToVert))) > 0) // check backface
        Out.Omni.rgb += (sqrt(intens) * omniRGBDelta[i].rgb * (1.0F - (sqrt(mag) * omniRGBDelta[i].w)));
  }
  Out.Omni.a = 1.0F;    
  //Out.Omni.rgb *= 0.5F;

  // Propagate color and texture coordinates:
  Out.Diff = inColor;
  Out.Tex.xy = inTex0;
  Out.Spec.xy = inTex0;
  Out.Spec.zw = inSkidmarks;

  return (Out);
}

