// Vertex shader for light flares 
// ------------------------------
// This shader makes the quad face the camera and change its size according to the camera vs quad normal.
// This shader should only be applied to quads (square consisting of 2 triangles) that face forward. 
//
// Created by Siim Annuk
// Date: June 28, 2009

#define USEFOG

float4x4 viewProjMatrix : register (c0);
float4x4 worldMatrix : register (c4);
float4x4 viewMatrix : register (c16);
float4x4 invViewMatrix : register (c20);
float4 worldEye: register (c8);
float4 fogData : register (c9);
float4 texMod : register (c11); // tex0.u=x, tex0.v=y, tex1.u=z, tex1.v=w
float4 specularColor : register (c15); // power in c15.a
float4 dirLight : register (c14);

struct VS_OUTPUT
{
   float4 Pos  : POSITION;
   float4 Diff : COLOR0;
#ifdef USEFOG
   float  Fog  : FOG;
#endif   
   float3 Tex  : TEXCOORD0;
};

VS_OUTPUT SSTR_SUN01 (float4 inPos : POSITION,
                      float2 inTex0 : TEXCOORD0, float2 inParam1 : TEXCOORD1,
                      float2 inParam2 : TEXCOORD2, float2 inParam3 : TEXCOORD3)
{
  VS_OUTPUT Out;

  float2 cornerOff = inTex0 * 2.0 - float2(1.0f, 1.0f);

float LAT = inParam2.x;
float NORTH = inParam2.y;

  float3x3 sunMatrix;
  sunMatrix[0] = float3(1.0, 0.0, 0.0);
  sunMatrix[1] = float3(0.0, sin(-LAT), cos(-LAT));
  sunMatrix[2] = float3(0.0, -cos(-LAT), sin(-LAT));

  float3x3 northMatrix;
  northMatrix[0] = float3(cos(NORTH), 0.0, -sin(NORTH));
  northMatrix[1] = float3(0.0, 1.0, 0.0);
  northMatrix[2] = float3(sin(NORTH), 0.0, cos(NORTH));

  sunMatrix = mul(sunMatrix, northMatrix);



float4 plane1;
plane1.xyz = sunMatrix[2];
plane1.w = inParam3.y;
float4 sunDir;
sunDir.xyz = normalize(dirLight.xyz);
sunDir.w = 1.0;

float d1 = dot(sunDir, plane1);

sunDir.y -= d1 / sunMatrix[2].y;
sunDir.xyz = normalize(sunDir.xyz);

////////////////////////////////////////


Out.Diff = float4(1.0, 1.0, 1.0, saturate(sunDir.y * 4.0 + 0.8));


  float3 eyeVector = normalize(mul(worldMatrix, sunDir.xyz) + worldEye);
  sunDir.xyz *= 900.0;

  float radius = 160.0f;
  
  float3 cornerOff2;
  float4x4 worldViewMatrix = mul(viewMatrix, worldMatrix);
  cornerOff2 = worldViewMatrix[0] * cornerOff.x + worldViewMatrix[1] * cornerOff.y;

  sunDir.xyz += cornerOff2 * radius;
  sunDir.w = 1.0;

  float4 outPos = mul (viewProjMatrix, sunDir);
  Out.Pos = outPos;


#ifdef USEFOG
  Out.Fog.x = 100000.0f;
#endif
  
  Out.Tex.xy = cornerOff * float2(0.5f, 0.5f) + float2(0.5f, 0.5f);
  Out.Tex.z = -0.2f;



  return (Out);
}








